print("import: 'watchdog'")
import watchdog

print("import: 'watchdog.observers'")
import watchdog.observers

print("import: 'watchdog.tricks'")
import watchdog.tricks

print("import: 'watchdog.utils'")
import watchdog.utils

